local enemy = {
	x = -100,
	y = 0,
	ct = 'enemyShip',	
	exploder = love.filesystem.load('explosions/explosion.lua')(),
	smoker = love.filesystem.load('smoke/smoke.lua')(),
	smoke = nil,
	fireSpeed = 1,
	charge = math.random(),
	projectiles = love.filesystem.load('projectiles/enemyprojectile.lua')(),
	speed = 30,
	destroyed = false,
	value = 100,
}

function enemy:setCollisionBody( body )
	enemy.body = body
	enemy.body.parent = enemy
end

function enemy:setMaxHealth( health )
	enemy.health = health
	enemy.maxHealth = health
end

function enemy:setImage( image )
	enemy.image = image
	enemy.width = enemy.image:getWidth()
	enemy.height = enemy.image:getHeight()	
end

function enemy:setLocation( x, y )
	self.x = x
	self.y = y	
end

function enemy:draw()
	-- draw shadow
	drawQueue:add( 5,
		function()
			love.graphics.setColor( 25, 25, 25, 255 )	
			love.graphics.draw( self.image, self.x-self.width/2 +20, self.y-self.height/2 +20)
			love.graphics.setColor( resetColor )
		end
	)
	
	-- draw ship
	drawQueue:add( 6, 
		function()
			love.graphics.draw( self.image, self.x-self.width/2, self.y-self.height/2 )
		end
	)
	
	drawQueue:add( 7, 
		function() 
			if self.smoke then
				love.graphics.draw( self.smoke )
			end
		end
	)
end

function enemy:cleanup()
	HC:remove( self.body )
	self.body = nil	
end

function enemy:takeDamage( damage )
	self.health = self.health - damage
	table.insert( entities, self.exploder:newExplosion(self.x + math.random(-self.width/2, self.width/2), self.y+ math.random(-self.height/2, self.height/2) ) )
	if not self.smoke and self.health * 2 < self.maxHealth then
		self.smoke = self.smoker:new( self.x, self.y, 0 )
	end
	if not self.destroyed and self.health <= 0 then
		self:death()
		self.destroyed = true
	end
end

function enemy:dead()
	return self.health <= 0 or self.y > viewportHeight + 50
end

function enemy:update( dt )
	if self.movement then
		self.movement( self, dt )
	else
		self.y = self.y + self.speed * dt
	end
	
	self.body:moveTo( self.x, self.y )
	if self.smoke then
		self.smoke:update( dt )
		self.smoke:setPosition( self.x, self.y )
	end
	self:chargeShot( dt )
end

function enemy:setMovement( func )
	self.movement = func
end

function enemy:death()
	if player then
		player:awardPoints( self.value )
	end
	for i = 1, 6 do
		table.insert( entities, self.exploder:newLargeExplosion(self.x + math.random(-self.width/2, self.width/2), self.y+ math.random(-self.height/2, self.height/2) ) )
	end
	
	local bonus = nil
	if math.random( 1, 10 ) == 1 then
		bonus = love.filesystem.load('powerups/healthbonus.lua')()
	else
		bonus = love.filesystem.load('powerups/cashbonus.lua')()		
	end
	bonus:setLocation( self.x, self.y )
	table.insert( entities, bonus )
	
end

function enemy:chargeShot( dt )
	self.charge = self.charge + dt
	if self.charge > self.fireSpeed then
		self.charge = self.charge - self.fireSpeed
		self:shoot()
	end
end

function enemy:shoot()
	self.projectiles:new( self.x, self.y )
end

return enemy