function love.load()

	hardon = require( "hardoncollider" )
	HC = hardon.new()
	HC:setCallbacks( on_collision )
	
	player = love.filesystem.load('player.lua')()
	entities = {}	
	table.insert( entities, love.filesystem.load('level.lua')() )
	table.insert( entities, love.filesystem.load('background.lua')() )
	table.insert( entities, love.filesystem.load('hud.lua')() )
	
	drawQueue = love.filesystem.load('drawQueue.lua')()
	
	resetColor = { 255, 255, 255, 255 }
	shadowColor = { 25, 25, 25, 255 }
	
	viewportWidth = 750
	viewportHeight = 600
	
	levelMoveSpeed = 20
	debug = false
end

function love.update(dt)

	fps = 1/dt
	global_dt = dt
	
	if player then
		player:update( dt )
		playerControls( dt )
	end
	if player and player:dead() then
		player:cleanup()
		player = nil
	end
		
	HC:update(dt)
	
	for i = #entities, 1, -1 do
		local entity = entities[i]
		entity:update( dt )
		if entity:dead() then
			entity:cleanup()
			table.remove( entities, i )
		end
	end
	
end

function love.draw()

	if player then
		player:draw()
	end
	for k, v in ipairs( entities ) do
		v:draw()
	end	
	drawQueue:flush()
	
	if debug then
		love.graphics.print( 'fps: ' .. fps, 50, 70 )
		love.graphics.print( 'entities: ' .. #entities, 50, 50 )
	end
	
end

function playerControls( dt ) 
	if love.keyboard.isDown('a') or love.keyboard.isDown('left') then
		player:left( dt )
	end
	
	if love.keyboard.isDown('d') or love.keyboard.isDown('right') then
		player:right( dt )
	end
	
	if love.keyboard.isDown('w') or love.keyboard.isDown('up') then
		player:up( dt )
	end
	
	if love.keyboard.isDown('s') or love.keyboard.isDown('down') then
		player:down( dt )
	end
	
	if love.keyboard.isDown( ' ' ) then
		player:fire( dt )
	end

	if love.keyboard.isDown( 'x' ) then
		player:fireSpecial( 1, dt )
	end
	
end

function on_collision( dt, shape_a, shape_b, mtv_x, mtv_y )

	if  shape_a.parent.ct == 'playerShip' and shape_b.parent.ct == 'enemyProjectile' then
		enemyScoredHit( shape_b.parent, shape_a.parent )
	elseif	shape_b.parent.ct == 'playerShip' and shape_a.parent.ct == 'enemyProjectile' then
		enemyScoredHit( shape_a.parent, shape_b.parent )
	elseif  shape_a.parent.ct == 'playerShip' and shape_b.parent.ct == 'enemyShip' then
		playerCollidesWithEnemy( shape_a.parent, shape_b.parent )
	elseif  shape_b.parent.ct == 'playerShip' and shape_a.parent.ct == 'enemyShip' then
		playerCollidesWithEnemy( shape_b.parent, shape_a.parent )
	elseif shape_a.parent.ct == 'playerProjectile' and shape_b.parent.ct == 'enemyShip' then
		playerScoredHit( shape_a.parent, shape_b.parent )
	elseif shape_b.parent.ct == 'playerProjectile' and shape_a.parent.ct == 'enemyShip' then
		playerScoredHit( shape_b.parent, shape_a.parent )
	elseif shape_a.parent.ct == 'playerShip' and shape_b.parent.ct == 'powerup' then
		playerGrabbedPowerup( shape_a.parent, shape_b.parent )
	elseif shape_b.parent.ct == 'playerShip' and shape_a.parent.ct == 'powerup' then
		playerGrabbedPowerup( shape_b.parent, shape_a.parent )
	end 

	
end

function playerScoredHit( projectile, enemy )
	projectile:resolve( enemy )
end

function enemyScoredHit( projectile, player )
	projectile:resolve( player )
end

function playerCollidesWithEnemy( player, enemy )
	player:takeDamage( 25 )
	enemy:takeDamage( 25 )
end

function playerGrabbedPowerup( player, powerup )
	powerup:pickup( player )
end