local player = {

	x = 400,
	y = 300,
	dX = 0,
	dY = 0,
	xAccel = 30,
	yAccel = 20,
	xMaxSpeed = 20,
	yMaxSpeed = 20,
	xFriction = 15,
	yFriction = 10,
	health = 100,
	ct = 'playerShip',
	exploder = love.filesystem.load('explosions/explosion.lua')(),
	score = 0,
	thrustFrame = 1,
	thrustAnim = 0,
	thrustAnimDuration = 0.03,
	thrustImages = {
		love.graphics.newImage('images/effects/thrusters/playerthruster1.png'),
		love.graphics.newImage('images/effects/thrusters/playerthruster2.png'),
		love.graphics.newImage('images/effects/thrusters/playerthruster3.png'),
		love.graphics.newImage('images/effects/thrusters/playerthruster4.png'),
		love.graphics.newImage('images/effects/thrusters/playerthruster5.png'),
		love.graphics.newImage('images/effects/thrusters/playerthruster6.png'),
		love.graphics.newImage('images/effects/thrusters/playerthruster7.png'),
		love.graphics.newImage('images/effects/thrusters/playerthruster8.png'),
	}
}

player.image = love.graphics.newImage( 'images/ships/player.png' )
player.width = player.image:getWidth()
player.height = player.image:getHeight()

player.body = HC:addRectangle( player.x, player.y, player.width, player.height )
player.body.parent = player

player.weapons = {}
local weapon = love.filesystem.load( 'weapons/weapon.lua' )()
weapon.ship = player
table.insert( player.weapons, weapon )
--[[
local weapon = love.filesystem.load( 'weapons/weapon2.lua' )()
weapon.ship = player
table.insert( player.weapons, weapon )
--]]

player.special = {{},{},{}}
local weapon = love.filesystem.load( 'weapons/barragepod.lua')()
weapon.ship = player
table.insert( player.special[1], weapon )

function player:left( dt )
	if self.dX > -self.xMaxSpeed then
		self.dX = self.dX - self.xAccel * dt
	end
end

function player:right( dt )
	if self.dX < self.xMaxSpeed then
		self.dX = self.dX + self.xAccel * dt
	end
end

function player:up( dt )
	if self.dY > -self.yMaxSpeed then
		self.dY = self.dY - self.yAccel * dt
	end
end

function player:down( dt )
	if self.dY < self.yMaxSpeed then
		self.dY = self.dY + self.yAccel * dt
	end
end

function player:fire( dt )
	for k,weapon in ipairs( self.weapons ) do
		weapon:fire( dt )
	end
end

function player:fireSpecial( bank, dt )
	for k, weapon in ipairs( self.special[ bank ] ) do
		weapon:fire( dt )
	end
end

function player:update(dt)

	player:friction( dt )

	self.x = self.x + self.dX
	self.y = self.y + self.dY
		
	player:clampToScreen()
	self.body:moveTo( self.x, self.y )
	
	for k,weapon in ipairs( self.weapons ) do
		weapon:update( dt )
	end	
	for k,bank in ipairs( self.special) do
		for k, weapon in ipairs( bank ) do
			weapon:update( dt )
		end
	end	
	
	player.thrustAnim = player.thrustAnim + dt
	if player.thrustAnim > player.thrustAnimDuration then
		player.thrustAnim = player.thrustAnim - player.thrustAnimDuration
		table.insert( player.thrustImages, 1, table.remove( player.thrustImages, 8 ) ) 
	end
end

function player:friction( dt )
	if math.abs( self.dX ) < self.xFriction * dt then
		self.dX = 0
	end
	if self.dX > 0 then
		self.dX = self.dX - self.xFriction * dt
	elseif self.dX < 0 then
		self.dX = self.dX + self.xFriction * dt
	end
	
	if math.abs( self.dY ) < self.yFriction * dt then
		self.dY = 0
	end
	if self.dY > 0 then
		self.dY = self.dY - self.yFriction * dt
	elseif self.dY < 0 then
		self.dY = self.dY + self.yFriction * dt
	end
end

function player:draw()
	--love.graphics.rectangle( 'fill', self.x-20, self.y-40, 40, 80 )
	
	drawQueue:add( 6, 	
		function() 			
			love.graphics.draw( self.image, self.x-self.width/2, self.y-self.height/2 )
		end )

	drawQueue:add( 5, 	
		function() 
			love.graphics.setColor( shadowColor )
			love.graphics.draw( self.image, self.x-self.width/2+20, self.y-self.height/2+20 )
			love.graphics.setColor( resetColor )
		end )
	
	for k,weapon in ipairs( self.weapons ) do
		weapon:draw()
	end
	for k,bank in ipairs( self.special) do
		for k, weapon in ipairs( bank ) do
			weapon:draw()
		end
	end	
	
	drawQueue:add( 6,
		function()
			love.graphics.draw( self.thrustImages[1], self.x-self.width/2, self.y + self.height/2 - 1, 0 , 1, 1 + (-self.dY / self.yMaxSpeed) )
		end
	)
end

function player:clampToScreen()
	
	if self.x > (viewportWidth - self.width/2) then
		self.x = viewportWidth - self.width/2
		self.dX = 0
	end
	if self.x < (0 + self.width/2) then
		self.x = 0 + self.width/2
		self.dX = 0
	end
	if self.y > (viewportHeight - self.height/2) then
		self.y = viewportHeight - self.height/2
		self.dY = 0
	end
	if self.y < (0 + self.height/2) then
		self.y = 0 + self.height/2
		self.dY = 0
	end	
end

function player:cleanup()
	HC:remove( self.body )
	self.body = nil
end

function player:takeDamage( damage )
	self.health = self.health - damage
	
	if self.health <= 50 then		
		table.insert( entities, self.exploder:newLargeExplosion(self.x + math.random(-self.width/2, self.width/2), self.y+ math.random(-self.height/2, self.height/2) ) )
	else 
		table.insert( entities, self.exploder:newExplosion(self.x + math.random(-self.width/2, self.width/2), self.y+ math.random(-self.height/2, self.height/2) ) )
	end
	
	if self.health <= 0 then
		self:death()
	end
end	

function player:death()
	for i = 1, 10 do
		table.insert( entities, self.exploder:newLargeExplosion(self.x + math.random(-self.width/2, self.width/2), self.y+ math.random(-self.height/2, self.height/2) ) )
	end
	for i = 1, 20 do
		table.insert( entities, self.exploder:newExplosion(self.x + math.random(-self.width/2, self.width/2), self.y+ math.random(-self.height/2, self.height/2) ) )
	end
	
end

function player:getHealthPerc()
	return self.health / 100
end

function player:addHealth( health )
	self.health = math.min( self.health + health, 100 )
	
end

function player:dead()
	return self.health <= 0
end

function player:awardPoints( points )
	player.score = player.score + points
end

return player