local projectile = {}

function projectile:new( x, y, direction )
	local newProjectile = {
		x = x,
		y = y,
		originX = x,
		originY = y,
		ct = 'playerProjectile',
		direction = direction,
		damage = 5,
		isDead = false,
		speed = 600,
		image = love.graphics.newImage( 'images/effects/rocket.png'),
		dying = false,
		drawDistance = 100,
		draw = function( self )
			if not self.dying then
				drawQueue:add( 6,
					function()
						love.graphics.draw( self.image, self.x + self.image:getWidth()/2, self.y + self.image:getHeight()/2, -self.direction, 1, 1, self.image:getWidth()/2, self.image:getHeight()/2 )
					end
				)
			end
			
			drawQueue:add( 5,
				function()
					love.graphics.setColor( 255, 255 ,255, 75 )
					local distance = math.sqrt( ( self.x - self.originX )^2 + ( self.y - self.originY ) ^2 )
					if distance > self.drawDistance then
						love.graphics.line( self.x + self.image:getWidth()/2, self.y + self.image:getHeight()/2, self.x+(self.originX - self.x)*( self.drawDistance / distance ), self.y+(self.originY - self.y)*( self.drawDistance / distance ) )
					else
						love.graphics.line( self.x + self.image:getWidth()/2, self.y + self.image:getHeight()/2, self.originX, self.originY )
					end
					love.graphics.setColor( resetColor )
				end
			)
		end,
		
		update = function( self, dt )
			if 	self.x < 0 or self.x > viewportWidth or
				self.y < 0 or self.y > viewportHeight then
					self.dying = true
			end

			if not self.dying then
				self.y = self.y - self.speed * math.cos( self.direction ) * dt
				self.x = self.x - self.speed * math.sin( self.direction ) * dt
				self.body:moveTo( self.x, self.y )
			else
				if self.body then
					HC:remove( self.body )
					self.body = nil
				end
			
				self.drawDistance = self.drawDistance - self.speed*dt
				if self.drawDistance <= 0 then
					self.isDead = true
				end
			end
		end,
		
		dead = function( self )
			return 	self.isDead					
		end,
		
		cleanup = function( self )
		end,
		
		resolve = function( self, enemy )
			enemy:takeDamage( self.damage )
			self.dying = true
		end
	}
	newProjectile.body = HC:addRectangle( x, y, newProjectile.image:getWidth(), newProjectile.image:getHeight() )
	newProjectile.body.parent = newProjectile
	
	table.insert( entities, newProjectile )
end

return projectile
